from __future__ import annotations

import typing

from . import _amulet_zlib, _version

__all__ = [
    "ZipBombException",
    "compiler_config",
    "compress_gzip",
    "compress_zlib",
    "decompress_zlib_gzip",
    "get_max_decompression_size",
    "set_max_decompression_size",
]

class ZipBombException(Exception):
    pass

def _init() -> None: ...
def compress_gzip(arg0: bytes) -> bytes:
    """
    Compress a bytes object using the gzip compression format.
    """

def compress_zlib(arg0: bytes) -> bytes:
    """
    Compress a bytes object using the zlib compression format.
    """

def decompress_zlib_gzip(arg0: bytes) -> bytes:
    """
    Decompress a bytes object compressed with either zlib or gzip.
    Raises ZipBombException if decompression requires more than the configured maximum decompression size.
    """

def get_max_decompression_size() -> int:
    """
    Get the configured maximum decompressed size in bytes. (Default 100MB)
    """

def set_max_decompression_size(arg0: typing.SupportsInt) -> None:
    """
    Set the configured maximum decompressed size in bytes.
    If decompression requires more memory than this it will raise ZipBombException.
    """

__version__: str
compiler_config: dict



import os
from packaging.version import Version

AMULET_COMPILER_TARGET_REQUIREMENT = "==2.0"
AMULET_COMPILER_VERSION_REQUIREMENT = "==3.0.0"

PYBIND11_REQUIREMENT = "==3.0.0"
AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = "~=1.1.0.0a0"

if os.environ.get("AMULET_PYBIND11_EXTENSIONS_REQUIREMENT", None):
    AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = f"{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT},{os.environ['AMULET_PYBIND11_EXTENSIONS_REQUIREMENT']}"


def get_specifier_set(version_str: str) -> str:
    """
    version_str: The PEP 440 version number of the library.
    """
    version = Version(version_str)
    if version.epoch != 0 or version.is_devrelease or version.is_postrelease:
        raise RuntimeError(f"Unsupported version format. {version_str}")

    return f"~={version.major}.{version.minor}.{version.micro}.0{''.join(map(str, version.pre or ()))}"


if os.environ.get("AMULET_FREEZE_COMPILER", None):
    import get_compiler

    AMULET_COMPILER_VERSION_REQUIREMENT = get_compiler.main()

    try:
        import amulet.pybind11_extensions
    except ImportError:
        pass
    else:
        AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = get_specifier_set(
            amulet.pybind11_extensions.__version__
        )


def get_build_dependencies() -> list:
    return [
        f"amulet-compiler-version{AMULET_COMPILER_VERSION_REQUIREMENT}",
        f"pybind11{PYBIND11_REQUIREMENT}",
        f"amulet-pybind11-extensions{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT}",
    ] * (not os.environ.get("AMULET_SKIP_COMPILE", None))


def get_runtime_dependencies() -> list[str]:
    return [
        f"amulet-compiler-target{AMULET_COMPILER_TARGET_REQUIREMENT}",
        f"amulet-compiler-version{AMULET_COMPILER_VERSION_REQUIREMENT}",
        f"pybind11{PYBIND11_REQUIREMENT}",
        f"amulet-pybind11-extensions{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT}",
    ]



from typing import TYPE_CHECKING
import logging as _logging

from . import _version

__version__ = _version.get_versions()["version"]

if TYPE_CHECKING:

    class ZipBombException(RuntimeError):
        pass

    def get_max_decompression_size() -> int: ...
    def set_max_decompression_size(max_decompression_size: int) -> None: ...

    def decompress_zlib_gzip(src: bytes) -> bytes: ...
    def compress_zlib(src: bytes) -> bytes: ...
    def compress_gzip(src: bytes) -> bytes: ...


# init a default logger
_logging.basicConfig(level=_logging.INFO, format="%(levelname)s - %(message)s")


def _init() -> None:
    import os
    import sys
    import ctypes

    if sys.platform == "win32":
        lib_path = os.path.join(os.path.dirname(__file__), "amulet_zlib.dll")
    elif sys.platform == "darwin":
        lib_path = os.path.join(os.path.dirname(__file__), "libamulet_zlib.dylib")
    elif sys.platform == "linux":
        lib_path = os.path.join(os.path.dirname(__file__), "libamulet_zlib.so")
    else:
        raise RuntimeError(f"Unsupported platform {sys.platform}")

    # Load the shared library
    ctypes.cdll.LoadLibrary(lib_path)

    from ._amulet_zlib import init

    init(sys.modules[__name__])


_init()
